@@ -9,7 +9,6 @@ import base64
 import threading
 import urllib.parse
 from itertools import chain, product
-from urllib.parse import urlencode
 
 from PIL import Image
 
@@ -294,6 +293,7 @@ class PkkAreaMerger(TileMerger, object):
         self.clear_code = clear_code
         self.extent = self.bbox
         self.area_type = area_type
+        self.use_cache = use_cache
 
         self.real_width = 0
         self.real_height = 0
@@ -325,16 +325,19 @@ class PkkAreaMerger(TileMerger, object):
                 file_name = "%s_%s%s" % (x, y, self.tile_format)
                 file_path = os.path.join(self.tile_dir, file_name)
 
-                imgstring = self.get_image(x, y)
-                if imgstring:
-                    tile = base64.b64decode(imgstring)
+                tries = 5
+                for i in range(1, tries + 1):
+                    try:
+                        img = self.get_image(x, y)
+                        if img:
+                            break
+                    except urllib.error.HTTPError as er:
+                        pass
+                if img:
+                    tile = base64.b64decode(img) if isinstance(img, str) else img
                     if tile:
                         self.write_image(tile, file_path)
                         self.count += 1
-                #     else:
-                #         self.log("Tile {} not loaded".format(file_path))
-                # else:
-                #     self.log("Tile {} not loaded".format(file_path))
 
                 if self.with_log:
                     print(
@@ -394,18 +397,27 @@ class PkkAreaMerger(TileMerger, object):
                 dx, dy = self.tile_size
             code = self.clear_code
 
-            # TODO: Understand how the parameter works.
-            # 6 10 no for -t 2
-            # layers = list(map(str, range(6, 10)))
-            # добавлен Layer: Кадастровые кварталы (ID: 20),
-            # ранняя версия генерировала ID от 0 до 19, что на некоторых ЗУ выдавало ошибку "Invalid 'layerDefs' is specified"
+            layerDefs = ""
+            # TODO: Understand how the layerDefs parameter works.
             if self.area_type == 10:
                 layers = [0, 1, 2, 6]
                 layerDefs = (
                     '{"0":"ID = \'%s\'","1":"objectid = -1","2":"objectid = -1","6":"objectid = -1"}'
                     % code
                 )
+            # elif self.area_type == 7:
+            #     layers = [0, 1, 5, 2, 6, 3, 7, 4]
 
+            #     # This formatting does not work for area_type = 1
+            #     format_layer_defs = "{"
+            #     id_format = [
+            #         ('"%s":"' % layer) + ("ID = '%s'" % code) + '"'
+            #         for layer in sorted(layers)
+            #     ]
+            #     format_layer_defs += ",".join(id_format)
+            #     format_layer_defs += "}"
+            #     safe_string = urllib.parse.quote_plus(format_layer_defs)
+            #     layerDefs = safe_string.replace("+", "%20")
             else:
                 layers = list(map(str, range(0, 21)))
                 layerDefs = {layer: str("ID = '{}'".format(code)) for layer in layers}
@@ -429,41 +441,50 @@ class PkkAreaMerger(TileMerger, object):
             url = self.url
             if self.area_type == 10:
                 url = url.replace("CadastreSelected", "ZONESSelected")
+            elif self.area_type == 7:
+                url = url.replace("CadastreSelected", "BordersGKNSelected")
 
-            url_parts = list(urllib.parse.urlparse(url))
-            query = dict(urllib.parse.parse_qsl(url_parts[4]))
-            query.update(params)
-            url_parts[4] = urlencode(query)
-            meta_url = urllib.parse.urlunparse(url_parts)
+            meta_url = url + "?" + urllib.parse.urlencode(params)
             if meta_url:
                 data = False
                 cache_path = os.path.join(self.tile_dir, "{}_{}.json".format(x, y))
-                try:
-                    with open(cache_path, "r") as data_file:
-                        data = json.loads(data_file.read())
-                        if not data.get("imageData") or not data.get("extent"):
-                            data = False
-                            raise Exception
-                except Exception:
-                    pass
-                try:
+                if self.use_cache:
+                    try:
+                        with open(cache_path, "r") as data_file:
+                            data = json.loads(data_file.read())
+                            if not data.get("imageData") or not data.get("extent"):
+                                data = False
+                    except Exception:
+                        # not in cache yet
+                        pass
+
                     if not data:
-                        response = self.make_request(meta_url)
-                        data = json.loads(response.decode("utf-8"))
-                        if data.get("imageData") and data.get("extent"):
-                            with open(cache_path, "w") as outfile:
-                                json.dump(data, outfile)
-                    if data and data.get("imageData"):
+                        data = self._load_image_data(meta_url, cache_path)
+                    if data:
                         self._image_extent_list.append(data.get("extent"))
-                        return data.get("imageData")
+                        if data.get("imageData"):
+                            return data.get("imageData")
                     else:
                         logger.warning("Can't get image meta data from: %s" % meta_url)
-                except Exception as er:
-                    logger.warning(er)
+
         elif not self.extent:
             logger.warning("Can't get image without extent")
         return False
 
+    def _load_image_data(self, url, cache_path):
+        response = self.make_request(url)
+        data = json.loads(response.decode("utf-8"))
+        if data.get("extent"):
+            if data.get("href"):
+                image_resp = self.make_request(data.get("href"))
+                if image_resp:
+                    base64_bytes = base64.b64encode(image_resp)
+                    base64_img = base64_bytes.decode("ascii")
+                    data["imageData"] = base64_img
+            if data.get("imageData"):
+                with open(cache_path, "w") as outfile:
+                    json.dump(data, outfile)
+
     def _merge_tiles(self):
         dx, dy = self._get_delta()
 
